@@ -1,6 +1,12 @@
+#' Prepare data for plotting by converting numeric to human-readable data
+#'
+#' @param data The data.frame to convert
+#' @return The data.frame with compartment names updated to character versions
+#'   and censoring indicating what type of censoring was used, if applicable
+#' @noRd
 .setupPlotData <- function(data) {
   .dat <- as.data.frame(data)
-  .w <- which(!is.na(.dat$RES))
+  .w <- which(!is.na(.dat$IRES))
   .dat <- .dat[.w, ]
   .doCmt <- FALSE
   if (any(names(.dat) == "CMT")) {
@@ -9,7 +15,7 @@
     }
   }
   if (!.doCmt) {
-    .dat$CMT <- factor(rep(1, length(.dat[, 1])), 1, "All Data")
+    .dat$CMT <- factor("All Data")
   } else {
     levels(.dat$CMT) <- paste("Endpoint: ", levels(.dat$CMT))
   }
@@ -17,7 +23,7 @@
     .censLeft <- any(.dat$CENS == 1)
     .censRight <- any(.dat$CENS == -1)
     if (.censLeft & .censRight) {
-      .dat$CENS <- factor(.dat$CENS, c(-1, 0, 1), c("Right censored data", "Observed data", "left censored data"))
+      .dat$CENS <- factor(.dat$CENS, c(-1, 0, 1), c("Right censored data", "Observed data", "Left censored data"))
     } else if (.censLeft) {
       .dat$CENS <- factor(.dat$CENS, c(0, 1), c("Observed data", "Censored data"))
     } else if (.censRight) {
@@ -29,9 +35,6 @@
   return(.dat)
 }
 
-#' @importFrom utils stack
-#' @importFrom ggplot2 aes element_blank facet_wrap geom_abline geom_point ggplot scale_color_manual theme scale_x_log10 scale_y_log10 xlab
-#' @importFrom rxode2 rxTheme
 .dvPlot <- function(.dat0, vars, log = FALSE) {
   .xgxr <- getOption("rxode2.xgxr", TRUE) &&
     requireNamespace("xgxr", quietly = TRUE)
@@ -48,7 +51,7 @@
       legend.title = ggplot2::element_blank()
     )
   } else {
-    dataPlot <- data.frame(DV = .dat0$DV, stack(.dat0[, vars, drop = FALSE]))
+    dataPlot <- data.frame(DV = .dat0$DV, utils::stack(.dat0[, vars, drop = FALSE]))
     .aes <- ggplot2::aes(.data$values, .data$DV)
     .color <- NULL
     .legendPos <- NULL
@@ -66,7 +69,7 @@
   }
   ggplot2::ggplot(dataPlot, .aes) +
     ggplot2::facet_wrap(~ind) +
-    ggplot2::geom_abline(slope = 1, intercept = 0, col = "red", size = 1.2) +
+    ggplot2::geom_abline(slope = 1, intercept = 0, col = "red", linewidth = 1.2) +
     .logx +
     .logy +
     ggplot2::geom_point(alpha = 0.5) +
@@ -123,25 +126,23 @@
 #' Plot some standard goodness of fit plots for the focei fitted object
 #'
 #' @param x a focei fit object
-#' @param ... additional arguments
-#' @return Nothing, called for its side effects
+#' @param ... additional arguments (currently ignored)
+#' @return An `nlmixr2PlotList` object (a list of ggplot2 objects with easier
+#'   plotting for all of them at the same time)
 #' @author Wenping Wang & Matthew Fidler
 #' @examples
 #' \donttest{
-#'
 #' library(nlmixr2est)
-#' ## The basic model consiss of an ini block that has initial estimates
 #' one.compartment <- function() {
 #'   ini({
-#'     tka <- 0.45 # Log Ka
-#'     tcl <- 1 # Log Cl
-#'     tv <- 3.45    # Log V
+#'     tka <- 0.45
+#'     tcl <- 1
+#'     tv <- 3.45
 #'     eta.ka ~ 0.6
 #'     eta.cl ~ 0.3
 #'     eta.v ~ 0.1
 #'     add.sd <- 0.7
 #'   })
-#'   # and a model block with the error sppecification and model specification
 #'   model({
 #'     ka <- exp(tka + eta.ka)
 #'     cl <- exp(tcl + eta.cl)
@@ -154,151 +155,143 @@
 #' }
 #'
 #' ## The fit is performed by the function nlmixr/nlmix2 specifying the model, data and estimate
-#' fit <- nlmixr2(one.compartment, theo_sd,  est="saem", saemControl(print=0))
+#' fit <- nlmixr2(one.compartment, theo_sd,  est="saem", saemControl(print=0, nBurn = 10, nEm = 20))
 #'
 #' # This shows many goodness of fit plots
 #' plot(fit)
-#'
 #' }
 #' @export
 plot.nlmixr2FitData <- function(x, ...) {
   .lst <- list()
   object <- x
   .tp <- traceplot(x)
-  if (!is.null(.tp)) .lst[[length(.lst) + 1]] <- .tp
+  if (!is.null(.tp)) {
+    .lst[["traceplot"]] <- .tp
+  }
   if (exists(".bootPlotData", object$env)) {
     .bp <- nlmixr2extra::bootplot(x)
-    .lst[[length(.lst) + 1]] <- .bp
+    .lst[["bootplot"]] <- .bp
   }
   .dat <- .setupPlotData(x)
-  .hasCwres <- any(names(.dat) == "CWRES")
-  .hasNpde <- any(names(.dat) == "NPD")
-  .hasPred <- any(names(.dat) == "PRED")
-  .hasIpred <- any(names(.dat) == "IPRED")
   for (.cmt in levels(.dat$CMT)) {
-    .dat0 <- .dat[.dat$CMT == .cmt,, drop = FALSE]
-    if (dim(.dat0)[1] > 0) {
-      if (.hasPred) {
-        .p1 <- .dvPlot(.dat0, c("PRED", "IPRED")) +
-          ggplot2::ggtitle(.cmt, "DV vs PRED/IPRED")
-        .lst[[length(.lst) + 1]] <- .p1
-
-        .p1 <- .dvPlot(.dat0, c("PRED", "IPRED"), TRUE) +
-          ggplot2::ggtitle(.cmt, "log-scale DV vs PRED/IPRED")
-        .lst[[length(.lst) + 1]] <- .p1
-      } else if (.hasIpred) {
-        .p1 <- .dvPlot(.dat0, "IPRED") +
-          ggplot2::ggtitle(.cmt, "DV vs IPRED")
-        .lst[[length(.lst) + 1]] <- .p1
-
-        .p1 <- .dvPlot(.dat0, "IPRED", TRUE) +
-          ggplot2::ggtitle(.cmt, "log-scale DV vs IPRED")
-        .lst[[length(.lst) + 1]] <- .p1
-      }
-
-
-      if (.hasCwres) {
-        .p1 <- .dvPlot(.dat0, c("CPRED", "IPRED")) +
-          ggplot2::ggtitle(.cmt, "DV vs CPRED/IPRED")
-        .lst[[length(.lst) + 1]] <- .p1
-
-        .p1 <- .dvPlot(.dat0, c("CPRED", "IPRED"), TRUE) +
-          ggplot2::ggtitle(.cmt, "log-scale DV vs CPRED/IPRED")
-        .lst[[length(.lst) + 1]] <- .p1
-      }
-
-      if (.hasNpde) {
-        .p1 <- .dvPlot(.dat0, c("EPRED", "IPRED")) +
-          ggplot2::ggtitle(.cmt, "DV vs EPRED/IPRED")
-        .lst[[length(.lst) + 1]] <- .p1
-
-        .p1 <- .dvPlot(.dat0, c("EPRED", "IPRED"), TRUE) +
-          ggplot2::ggtitle(.cmt, "log-scale DV vs EPRED/IPRED")
-        .lst[[length(.lst) + 1]] <- .p1
-      }
-
-      for (x in c("IPRED", "PRED", "CPRED", "EPRED", "TIME", "tad")) {
-        if (any(names(.dat0) == x)) {
-          for (y in c("IWRES", "IRES", "RES", "CWRES", "NPD")) {
-            if (any(names(.dat0) == y)) {
-              if (y == "CWRES" && x %in% c("TIME", "CPRED")) {
-                .doIt <- TRUE
-              } else if (y == "NPD" && x %in% c("TIME", "EPRED")) {
-                .doIt <- TRUE
-              } else if (!(y %in% c("CWRES", "NPD"))) {
-                .doIt <- TRUE
-              }
-              if (.doIt) {
-                .p2 <- .scatterPlot(.dat0, c(x, y), .cmt, log = FALSE)
-                .lst[[length(.lst) + 1]] <- .p2
-                .p2 <- .scatterPlot(.dat0, c(x, y), .cmt, log = TRUE)
-                .lst[[length(.lst) + 1]] <- .p2
-              }
-            }
-          }
-        }
-      }
-      # .idPlot <- try(plot.nlmixr2AugPred(nlmixr2AugPred(object)));
-      # if (inherits(.idPlot, "try-error")){
-      .ids <- unique(.dat0$ID)
-      .s <- seq(1, length(.ids), by = 16)
-      .j <- 0
-      for (i in .s) {
-        .j <- .j + 1
-        .tmp <- .ids[seq(i, i + 15)]
-        .tmp <- .tmp[!is.na(.tmp)]
-        .d1 <- .dat0[.dat0$ID %in% .tmp, ]
-
-        .p3 <- ggplot2::ggplot(.d1, ggplot2::aes(x = .data$TIME, y = .data$DV)) +
-          ggplot2::geom_point() +
-          ggplot2::geom_line(aes(x = .data$TIME, y = .data$IPRED), col = "red", size = 1.2)
-        if (any(names(.d1) == "PRED")) {
-          .p3 <- .p3 + ggplot2::geom_line(aes(x = .data$TIME, y = .data$PRED), col = "blue", size = 1.2)
-        }
-        .p3 <- .p3 + ggplot2::facet_wrap(~ID) +
-          ggplot2::ggtitle(.cmt, sprintf("Individual Plots (%s of %s)", .j, length(.s))) +
-          rxode2::rxTheme()
-        if (any(names(.d1) == "lowerLim")) {
-          .p3 <-
-            .p3 +
-            geom_cens(aes(lower = .data$lowerLim, upper = .data$upperLim), fill = "purple")
-        }
-        .lst[[length(.lst) + 1]] <- .p3
-      }
-      .dat0$id2 <- factor(paste0("id: ", .dat0$ID, "; dose#: ", .dat0$dosenum))
-      .ids <- unique(.dat0$id2)
-      .s <- seq(1, length(.ids), by = 16)
-      .j <- 0
-      # for (i in .s) {
-      #   .j <- .j + 1
-      #   .tmp <- .ids[seq(i, i + 15)]
-      #   .tmp <- .tmp[!is.na(.tmp)]
-      #   .d1 <- .dat0[.dat0$id2 %in% .tmp, ]
-
-      #   .p3 <- ggplot2::ggplot(.d1, ggplot2::aes(x = tad, y = DV)) +
-      #     ggplot2::geom_point() +
-      #     ggplot2::geom_line(aes(x = tad, y = IPRED), col = "red", size = 1.2) +
-      #     ggplot2::geom_line(aes(x = tad, y = PRED), col = "blue", size = 1.2) +
-      #     ggplot2::facet_wrap(~id2) +
-      #     ggplot2::ggtitle(.cmt, sprintf("Individual TAD Plots (%s of %s)", .j, length(.s))) +
-      #     rxode2::rxTheme()
-      #   if (any(names(.d1) == "lowerLim")) {
-      #     .p3 <- .p3 + geom_cens(aes(lower=lowerLim, upper=upperLim), fill="purple")
-      #   }
-      #   .lst[[length(.lst) + 1]] <- .p3
-      # }
-    }
+    .lst[[.cmt]] <- plotCmt(.dat, cmt = .cmt)
   }
 
-  # .id <- unique(.dat0$id2)
-  # if (grDevices::dev.cur() != 1){
-  #     .x  <- .lst
-  #     for (.i in seq_along(.x)){
-  #         plot(.x[[.i]])
-  #     }
-  # }
   class(.lst) <- "nlmixr2PlotList"
-  return(.lst)
+  .lst
+}
+
+#' Plot data from one compartment
+#'
+#' @inheritParams plot.nlmixr2FitData
+#' @param cmt The value of the current compartment
+#' @return A list of ggplot2 objects
+#' @noRd
+plotCmt <- function(x, cmt) {
+  .lst <- list()
+  .hasCwres <- any(names(x) == "CWRES")
+  .hasNpde <- any(names(x) == "NPD")
+  .hasPred <- any(names(x) == "PRED")
+  .hasIpred <- any(names(x) == "IPRED")
+  .datCmt <- x[x$CMT == cmt,, drop = FALSE]
+  if (nrow(.datCmt) > 0) {
+    if (.hasPred & .hasIpred) {
+      .lst[["dv_pred_ipred_linear"]] <-
+        .dvPlot(.datCmt, c("PRED", "IPRED")) +
+        ggplot2::ggtitle(cmt, "DV vs PRED/IPRED")
+
+      .lst[["dv_pred_ipred_log"]] <-
+        .dvPlot(.datCmt, c("PRED", "IPRED"), TRUE) +
+        ggplot2::ggtitle(cmt, "log-scale DV vs PRED/IPRED")
+    } else if (.hasIpred) {
+      .lst[["dv_ipred_linear"]] <-
+        .dvPlot(.datCmt, "IPRED") +
+        ggplot2::ggtitle(cmt, "DV vs IPRED")
+
+      .lst[["dv_ipred_log"]] <-
+        .dvPlot(.datCmt, "IPRED", TRUE) +
+        ggplot2::ggtitle(cmt, "log-scale DV vs IPRED")
+    } else if (.hasPred) {
+      .lst[["dv_pred_linear"]] <-
+        .dvPlot(.datCmt, "PRED") +
+        ggplot2::ggtitle(cmt, "DV vs PRED")
+
+      .lst[["dv_pred_log"]] <-
+        .dvPlot(.datCmt, "PRED", TRUE) +
+        ggplot2::ggtitle(cmt, "log-scale DV vs PRED")
+    }
+
+    if (.hasCwres) {
+      .lst[["dv_cpred_linear"]] <-
+        .dvPlot(.datCmt, c("CPRED", "IPRED")) +
+        ggplot2::ggtitle(cmt, "DV vs CPRED/IPRED")
+
+      .lst[["dv_cpred_log"]] <-
+        .dvPlot(.datCmt, c("CPRED", "IPRED"), TRUE) +
+        ggplot2::ggtitle(cmt, "log-scale DV vs CPRED/IPRED")
+    }
+
+    if (.hasNpde) {
+      .lst[["dv_epred_linear"]] <-
+        .dvPlot(.datCmt, c("EPRED", "IPRED")) +
+        ggplot2::ggtitle(cmt, "DV vs EPRED/IPRED")
+
+      .lst[["dv_epred_log"]] <-
+        .dvPlot(.datCmt, c("EPRED", "IPRED"), TRUE) +
+        ggplot2::ggtitle(cmt, "log-scale DV vs EPRED/IPRED")
+    }
+
+    for (x in intersect(names(.datCmt), c("IPRED", "PRED", "CPRED", "EPRED", "TIME", "tad"))) {
+      for (y in intersect(names(.datCmt), c("IWRES", "IRES", "RES", "CWRES", "NPD"))) {
+        if (y == "CWRES" && x %in% c("TIME", "CPRED")) {
+          .doIt <- TRUE
+        } else if (y == "NPD" && x %in% c("TIME", "EPRED")) {
+          .doIt <- TRUE
+        } else if (!(y %in% c("CWRES", "NPD"))) {
+          .doIt <- TRUE
+        }
+        if (.doIt) {
+          .lst[[paste(y, x, "linear", sep = "_")]] <-
+            .scatterPlot(.datCmt, c(x, y), cmt, log = FALSE)
+          .lst[[paste(y, x, "log", sep = "_")]] <-
+            .scatterPlot(.datCmt, c(x, y), cmt, log = TRUE)
+        }
+      }
+    }
+    # .idPlot <- try(plot.nlmixr2AugPred(nlmixr2AugPred(object)));
+    # if (inherits(.idPlot, "try-error")){
+    .ids <- unique(.datCmt$ID)
+    .s <- seq(1, length(.ids), by = 16)
+    .j <- 0
+    for (i in .s) {
+      .j <- .j + 1
+      .tmp <- .ids[seq(i, i + 15)]
+      .tmp <- .tmp[!is.na(.tmp)]
+      .d1 <- .datCmt[.datCmt$ID %in% .tmp, ]
+
+      .pIndividual <- ggplot2::ggplot(.d1, ggplot2::aes(x = .data$TIME, y = .data$DV)) +
+        ggplot2::geom_point() +
+        ggplot2::geom_line(ggplot2::aes(x = .data$TIME, y = .data$IPRED), col = "red", linewidth = 1.2)
+      if (any(names(.d1) == "PRED")) {
+        .pIndividual <- .pIndividual + ggplot2::geom_line(ggplot2::aes(x = .data$TIME, y = .data$PRED), col = "blue", linewidth = 1.2)
+      }
+      .pIndividual <- .pIndividual + ggplot2::facet_wrap(~ID) +
+        ggplot2::ggtitle(cmt, sprintf("Individual Plots (%s of %s)", .j, length(.s))) +
+        rxode2::rxTheme()
+      if (any(names(.d1) == "lowerLim")) {
+        .pIndividual <-
+          .pIndividual +
+          geom_cens(ggplot2::aes(lower = .data$lowerLim, upper = .data$upperLim), fill = "purple")
+      }
+      .lst[[paste("individual", i, sep = "_")]] <- .pIndividual
+    }
+    .datCmt$id2 <- factor(paste0("id: ", .datCmt$ID, "; dose#: ", .datCmt$dosenum))
+    .ids <- unique(.datCmt$id2)
+    .s <- seq(1, length(.ids), by = 16)
+    .j <- 0
+  }
+  class(.lst) <- "nlmixr2PlotList"
+  .lst
 }
 
 #' @export
@@ -320,10 +313,9 @@ plot.nlmixr2FitCore <- function(x, ...) {
   stop("This is not a nlmixr2 data frame and cannot be plotted")
 }
 
-##' @export
+#' @export
 plot.nlmixr2FitCoreSilent <- plot.nlmixr2FitCore
 
-#'
 #' @title Produce trace-plot for fit if applicable
 #'
 #' @param x fit object
@@ -382,7 +374,7 @@ traceplot.nlmixr2FitCore <- function(x, ...) {
     if (!is.null(.niter)) {
       .p0 <-
         .p0 +
-        ggplot2::geom_vline(xintercept = .niter, col = "blue", size = 1.2)
+        ggplot2::geom_vline(xintercept = .niter, col = "blue", linewidth = 1.2)
     }
     .p0 <- .p0 + rxode2::rxTheme()
     return(.p0)
@@ -391,5 +383,5 @@ traceplot.nlmixr2FitCore <- function(x, ...) {
   }
 }
 
-##' @export
+#' @export
 traceplot.nlmixr2FitCoreSilent <- traceplot.nlmixr2FitCore
